package eu.mhsl.minecraft.pixelblocks.pixelblock;

import eu.mhsl.minecraft.pixelblocks.Main;
import eu.mhsl.minecraft.pixelblocks.utils.ListUtil;
import org.bukkit.Location;
import org.bukkit.NamespacedKey;
import org.bukkit.block.BlockFace;
import org.bukkit.block.data.Directional;
import org.bukkit.entity.BlockDisplay;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.persistence.PersistentDataType;
import org.bukkit.util.Transformation;
import org.bukkit.util.Vector;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.joml.Vector3d;

import java.util.List;
import java.util.Objects;

public class Pixels {
    private static final NamespacedKey pixelOfTag = new NamespacedKey(Main.plugin(), "pixel_of");

    private final @NotNull PixelBlock parentBlock;

    public Pixels(@NotNull PixelBlock parentBlock) {
        this.parentBlock = parentBlock;
    }

    public void spawn() {
        ListUtil.splitListInParts(10, this.parentBlock.getPixelData())
            .forEach(pixels -> parentBlock.getBlockTaskChain()
                .delay(1)
                .sync(() -> pixels.forEach(this::spawnPixel))
                .execute());
    }

    private void spawnPixel(PixelBlockWorld.PixelData pixelData) {
        Location pixelBlockLocation = this.parentBlock.getPixelBlockLocation();
        Location pixelLocation = pixelBlockLocation.add(pixelData.relativeLocation().multiply(pixelData.scale()));
        Vector3d centerOffset = new Vector3d(0.5, 0.5, 0.5).mul(pixelData.scale());
        BlockDisplay entity = (BlockDisplay) pixelBlockLocation.getWorld().spawnEntity(
            pixelLocation.add(Vector.fromJOML(centerOffset)),
            EntityType.BLOCK_DISPLAY
        );

        entity.setBlock(pixelData.block());
        this.setEntityRotation(entity, pixelData.directional());

        Transformation transform = entity.getTransformation();
        transform.getScale().set(pixelData.scale());
        transform.getTranslation().set(centerOffset.mul(-1));
        entity.setTransformation(transform);

        entity.getPersistentDataContainer().set(pixelOfTag, PersistentDataType.STRING, this.parentBlock.getBlockUUID().toString());
    }

    public void destroy() {
        List<BlockDisplay> entities = parentBlock.getPixelBlockLocation().getNearbyEntitiesByType(BlockDisplay.class, 1)
            .stream()
            .filter(blockDisplay -> blockDisplay.getPersistentDataContainer().has(pixelOfTag))
            .filter(blockDisplay -> Objects.equals(
                blockDisplay.getPersistentDataContainer().get(pixelOfTag, PersistentDataType.STRING),
                parentBlock.getBlockUUID().toString()
            ))
            .toList();

        ListUtil.splitListInParts(10, entities)
            .forEach(pixels -> parentBlock.getBlockTaskChain()
                .delay(1)
                .sync(() -> pixels.forEach(Entity::remove))
                .execute());
    }

    private void setEntityRotation(Entity entity, @Nullable Directional direction) {
        if(direction == null) return;
        BlockFace blockFace = direction.getFacing();
        float yaw = switch (blockFace) {
            case NORTH -> 0;
            case SOUTH -> 180;
            case WEST -> -90;
            case EAST -> 90;
            default -> entity.getLocation().getYaw();
        };

        float pitch = switch (blockFace) {
            case UP -> -90;
            case DOWN -> 90;
            default -> 0;
        };
        entity.setRotation(yaw, pitch);
    }
}
